!=============================================================================================
program checIcs
   !# program to verify the Intial conditions generate by pre-brams
   !#
   !# @note
   !#
   !# **Documentation/Version Control**: <no doc>
   !#
   !# **Author(s)**: Luiz Flávio Rodrigues **e-mail:** <luiz.rodrigues@inpe.br>
   !#
   !# **Date**: 19 August 2021 (Thursday)
   !#
   !# **Full Description**: program to verify the Intial conditions generate by pre-brams
   !#
   !# @endnote
   !#
   !# @warning
   !# ![](https://licensebuttons.net/l/by-sa/3.0/88x31.png"")
   !# Now is under CC Attribution-ShareAlike 4.0 International, please see:
   !# &copy; <https://creativecommons.org/licenses/by-sa/4.0/>
   !# @endwarning
   !#
    
   !Use area
   use dump !Dump contains a lot of functions for debugs and formated printouts
   use memoryMod, only: & ! The list of namelist variables to be read
                        graFileNames,ctlFileNames &
                        ,ctlInfo,maxValue,minValue
   use filesMod, only:  readFirstCtl, readAllGrads, makeTable
   use utilsMod, only: getUnit, releaseUnit, fileExist

   implicit none

   include "constants.h"
   character(len=*),parameter :: sourceName='main.F90' !Name of this source code
   character(len=*),parameter :: procedureName='**checIcs**' !Name of this procedure
   !
   !Local Parameters

   !Input/Output variables

   !Local variables
   integer :: stepsBetDates, lUnit, fCount
   character(:), allocatable :: arg
   integer arglen, stat

   !Code

   !Getting arguments (dir) and generating a list of files (gra and ctl)
   call get_command_argument(number=1, length=arglen)  ! Assume for simplicity success
   allocate (character(arglen) :: arg)
   call get_command_argument(number=1, value=arg, status=stat)

   call system('ls '//arg//'*.ctl > ctlContents.txt')
   call system('ls '//arg//'*.gra > graContents.txt')
   call system('wc -l graContents.txt > lineCount.txt')

   !Open file, get the number of files and allocate the fileNames arrays
   if(.not. fileExist('lineCount.txt')) iErrNumber=dumpMessage(c_tty,c_yes,sourceName,procedureName &
              ,c_fatal,'File lineCount.txt not found. May be the system have problem. Please check it!')
   lunit=getUnit()
   open(unit=lunit, file=trim('lineCount.txt'), form='formatted', status='old')
   read(lUnit,*) stepsBetDates
   ierrNumber=releaseUnit(lunit)
   allocate(ctlFileNames(stepsBetDates), graFileNames(stepsBetDates))

   !Open file of ctl names and get the fileNames
   if(.not. fileExist('ctlContents.txt')) iErrNumber=dumpMessage(c_tty,c_yes,sourceName,procedureName &
              ,c_fatal,'File ctlContents.txt not found. Really? What?. Please check it!')
   lunit=getUnit()
   open(unit=lunit, file=trim('ctlContents.txt'), form='formatted', status='old')
   do fCount=1,stepsBetDates
      read(lUnit,fmt='(A)') ctlFileNames(fCount)
   enddo
   ierrNumber=releaseUnit(lunit) 

   !Open file of gra names and get the fileNames
   if(.not. fileExist('graContents.txt')) iErrNumber=dumpMessage(c_tty,c_yes,sourceName,procedureName &
              ,c_fatal,'File graContents.txt not found. Really? What?. Please check it!')
   lunit=getUnit()
   open(unit=lunit, file=trim('graContents.txt'), form='formatted', status='old')
   do fCount=1,stepsBetDates
      read(lUnit,fmt='(A)') graFileNames(fCount)
   enddo
   ierrNumber=releaseUnit(lunit) 

   call system('rm graContents.txt lineCount.txt ctlContents.txt')

   call readFirstCtl(ctlFileNames(1),ctlInfo)

   allocate(maxValue(ctlInfo%nVars,ctlInfo%zDef,stepsBetDates), minValue(ctlInfo%nVars,ctlInfo%zDef,stepsBetDates))

   call readAllGrads(ctlInfo,stepsBetDates,maxValue,minValue,graFileNames)

   call makeTable(ctlInfo,stepsBetDates,maxValue,minValue,graFileNames)

end program checIcs 