!=============================================================================================
module filesMod
   !# This module contains subroutines and functions to access files
   !#
   !# @note
   !#
   !# **Documentation/Version Control**: <no doc>
   !#
   !# **Author(s)**: Luiz Flávio Rodrigues **e-mail:** <luiz.rodrigues@inpe.br>
   !#
   !# **Date**: 19 August 2021 (Thursday)
   !#
   !# **Full Description**: This module contains subroutines and functions to access files
   !#
   !# @endnote
   !#
   !# @warning
   !# ![](https://licensebuttons.net/l/by-sa/3.0/88x31.png"")
   !# Now is under CC Attribution-ShareAlike 4.0 International, please see:
   !# &copy; <https://creativecommons.org/licenses/by-sa/4.0/>
   !# @endwarning
   !#
    
   !Use area
   use dump !Dump contains a lot of functions for debugs and formated printouts

   use utilsMod, only: &
                  getUnit, &
                  releaseUnit, &
                  fileExist, &
                  date_add_to_dble, &
                  outRealSize

   use memoryMod, only: ctli

   implicit none

   include "constants.h"
   character(len=*),parameter :: sourceName='filesMod.F90' !Name of this source code
   character(len=*),parameter :: procedureName='**fileMod**' !Name of this procedure
   !
   !Local Parameters

   !Local variables

   contains

   !=============================================================================================
   subroutine readFirstCtl(ctlFileName,ctlInfo)
      !# Read the forst ctl ang get the file information
      !#
      !# @note
      !#
      !# **Documentation/Version Control**: <no doc>
      !#
      !# **Author(s)**: Luiz Flávio Rodrigues **e-mail:** <luiz.rodrigues@inpe.br>
      !#
      !# **Date**: 19 August 2021 (Thursday)
      !#
      !# **Full Description**: Read the forst ctl ang get the file information
      !#
      !# @endnote
      !#
      !# @warning
      !# ![](https://licensebuttons.net/l/by-sa/3.0/88x31.png"")
      !# Now is under CC Attribution-ShareAlike 4.0 International, please see:
      !# &copy; <https://creativecommons.org/licenses/by-sa/4.0/>
      !# @endwarning
      !#
       
      !Use area
   
      implicit none
   
      character(len=*),parameter :: procedureName='**readFirstCtl**' !Name of this procedure
      !
      !Local Parameters
   
      !Input/Output variables
      character(len=*), intent(in) :: ctlFileName
      type(ctli) :: ctlInfo !intent(out)
   
      !Local variables
      integer :: lUnit, z
      character(len=256) :: lixo
      character(len=3) :: cz
      real :: lev(100)
   
      !Code
      if(.not. fileExist(ctlFileName)) iErrNumber=dumpMessage(c_tty,c_yes,sourceName,procedureName &
              ,c_fatal,'File '//ctlFileName &
              //' not found. Please, verify and solve it!')
      lunit=getUnit()
      open(unit=lunit, file=trim(ctlFileName), form='formatted', status='old')
      read(lUnit,*) lixo
      read(lUnit,*) lixo
      read(lUnit,*) lixo
      read(lUnit,*) lixo,ctlInfo%xDef,lixo,ctlInfo%lonI,ctlInfo%deltaLon
      read(lUnit,*) lixo,ctlInfo%yDef,lixo,ctlInfo%latI,ctlInfo%deltaLat
      read(lUnit,*) lixo,ctlInfo%zDef
      allocate(ctlInfo%levels(ctlInfo%zDef))
      write(cz,fmt='(I3.3)') ctlInfo%zDef
      rewind(unit=lUnit)
      read(lUnit,*) lixo
      read(lUnit,*) lixo
      read(lUnit,*) lixo
      read(lUnit,*) lixo
      read(lUnit,*) lixo
      read(lUnit,*) lixo,ctlInfo%zDef,lixo,ctlInfo%levels
      read(lUnit,*) lixo
      read(lUnit,*) lixo,ctlInfo%nVars 
      allocate(ctlInfo%varNames(ctlInfo%nVars))
      do z=1,ctlInfo%nVars
         read(lUnit,*) ctlInfo%varNames(z)
      enddo

      !read(lunit, args_input)
      ierrNumber=releaseUnit(lunit)

      write (*,*) 'File Inventory: '
      write (*,*) '============================='
      write (*,*) 'xDef       :   ',ctlInfo%xDef          
      write (*,*) 'yDef       :   ',ctlInfo%yDef          
      write (*,*) 'zDef       :   ',ctlInfo%zDef          
      write (*,*) 'loni       :   ',ctlInfo%loni          
      write (*,*) 'lati       :   ',ctlInfo%lati          
      write (*,*) 'deltaLon   :   ',ctlInfo%deltaLon      
      write (*,*) 'deltaLat   :   ',ctlInfo%deltaLat      
      write (*,fmt='(A,'//cz//'(F6.1,1X))') ' levels(:)  :   ',ctlInfo%levels     
      write (*,*) 'nvars      :   ',ctlInfo%nvars         
      write (*,*) 'varNames(:):   ',ctlInfo%varNames   

   
   end subroutine readFirstCtl 

   !=============================================================================================
   subroutine  readAllGrads(ctlInfo,stepsBetDates,maxValue,minValue,graFileNames)
      !# Read and compute max and mins for each file, var and level
      !#
      !# @note
      !#
      !# **Documentation/Version Control**: <no doc>
      !#
      !# **Author(s)**: Luiz Flávio Rodrigues **e-mail:** <luiz.rodrigues@inpe.br>
      !#
      !# **Date**: 19 August 2021 (Thursday)
      !#
      !# **Full Description**: Read and compute max and mins for each file, var and level
      !#
      !# @endnote
      !#
      !# @warning
      !# ![](https://licensebuttons.net/l/by-sa/3.0/88x31.png"")
      !# Now is under CC Attribution-ShareAlike 4.0 International, please see:
      !# &copy; <https://creativecommons.org/licenses/by-sa/4.0/>
      !# @endwarning
      !#
       
      !Use area
   
      implicit none
   
      character(len=*),parameter :: procedureName='**readAllGrads**' !Name of this procedure
      !
      !Local Parameters
   
      !Input/Output variables
      type(ctli) :: ctlInfo !intent(in)
      integer, intent(in) :: stepsBetDates
      real, intent(out) :: maxValue(ctlInfo%nVars,ctlInfo%zDef,stepsBetDates)
      real, intent(out) :: minValue(ctlInfo%nVars,ctlInfo%zDef,stepsBetDates)
      character(len=*), intent(in) :: graFileNames(stepsBetDates)
   
      !Local variables
      integer :: nStep, irec, nvar, nnz, lUnit
      integer :: recordLen
      real :: values(ctlInfo%xDef,ctlInfo%yDef)
   
      !Code
      recordLen=outRealSize()*ctlInfo%xDef*ctlInfo%yDef
      do nStep=1,stepsBetDates

         if(.not. fileExist(trim(graFileNames(nStep)))) then
               iErrNumber=dumpMessage(c_tty,c_yes,sourceName,procedureName &
              ,c_warning,'File '//trim(graFileNames(nStep)) &
              //' not found. Please, verify and solve it!')
               cycle
         endif
         lunit=getUnit()
         !write(*,*) "====================================================",nStep,lUnit
         open(unit=lunit,file=trim(graFileNames(nStep)),action='READ',status='OLD' &
             ,form='UNFORMATTED',access='DIRECT',recl=recordLen)
         
         write(*,*) 'Reading ',trim(graFileNames(nStep))
         irec=1
         do nvar=1,ctlInfo%nVars
            !write (*,fmt='(A16,1X," : ")',advance='no') trim(ctlInfo%varNames(nvar))
            do nnz=1,ctlInfo%zDef
               !write (*,fmt='(I3.3,1X," : ")',advance='no') nnz
               read(lunit,rec=irec) values
               maxValue(nvar,nnz,nStep)=maxval(values)
               minvalue(nvar,nnz,nStep)=minval(values)
               !if (trim(ctlInfo%varNames(nvar))=='RH' .or. trim(ctlInfo%varNames(nvar))=='UGRD'     &
               !   .or.  trim(ctlInfo%varNames(nvar))=='VGRD' .or. trim(ctlInfo%varNames(nvar))=='TMP') then
               !   write (*,fmt='(A16,1X,": ","lev=",F6.1,1X,F8.1,1X,F8.1)') trim(ctlInfo%varNames(nvar)),ctlInfo%levels(nnz),minvalue(nvar,nnz,nStep),maxValue(nvar,nnz,nStep)
               !else
               !   write (*,fmt='(A16,1X,": ","lev=",F6.1,1X,F8.1,1X,F8.1)') trim(ctlInfo%varNames(nvar)),ctlInfo%levels(nnz),minvalue(nvar,nnz,nStep),maxValue(nvar,nnz,nStep)
               !endif
               irec=irec+1
            enddo
            !write (*,*) ''
        enddo
        ierrNumber=releaseUnit(lunit)

      enddo 
   
   end subroutine  readAllGrads 

   !=============================================================================================
   subroutine makeTable(ctlInfo,stepsBetDates,maxValue,minValue,graFileNames)
      !# Draw a table with information
      !#
      !# @note
      !#
      !# **Documentation/Version Control**: <no doc>
      !#
      !# **Author(s)**: Luiz Flávio Rodrigues **e-mail:** <luiz.rodrigues@inpe.br>
      !#
      !# **Date**: 19 August 2021 (Thursday)
      !#
      !# **Full Description**: Draw a table with information
      !#
      !# @endnote
      !#
      !# @warning
      !# ![](https://licensebuttons.net/l/by-sa/3.0/88x31.png"")
      !# Now is under CC Attribution-ShareAlike 4.0 International, please see:
      !# &copy; <https://creativecommons.org/licenses/by-sa/4.0/>
      !# @endwarning
      !#
       
      implicit none

      character(len=*),parameter :: procedureName='**makeTable**' !Name of this procedure
      !
      !Local Parameters
   
      !Input/Output variables
      type(ctli) :: ctlInfo !intent(in)
      integer, intent(in) :: stepsBetDates
      real, intent(in) :: maxValue(ctlInfo%nVars,ctlInfo%zDef,stepsBetDates)
      real, intent(in) :: minValue(ctlInfo%nVars,ctlInfo%zDef,stepsBetDates)
      character(len=*), intent(in) :: graFileNames(stepsBetDates)

      !Local variables
      integer :: iFile,iLev,lUnit,csvUnit,iVar
      character(len=3) :: cl

      lunit=getUnit()
      open(unit=lunit, file='ciTable.txt', form='formatted', action='write', status='replace')

      !Code
      do iFile=1,stepsBetDates
         write (lUnit,*) ''
         write (lUnit,fmt='(A)') '+------------------------------------------------------------------------------------------------+'
         write (lUnit,fmt='(A1,A96,A1)') '|',graFileNames(iFile),'|'
         write (lUnit,fmt='(A)') '+------+-----------------+-----------------+-----------------+-----------------+-----------------+'
         write (lUnit,fmt='("|",A6,"|",5(A17,"|"))') 'Level',ctlInfo%varnames(:)
         write (lUnit,fmt='(A)') '+------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+'
         write (lUnit,fmt='(A)') '| mBar |   Min  :   Max  |   Min  :   Max  |   Min  :   Max  |   Min  :   Max  |   Min  :   Max  |'
         write (lUnit,fmt='(A)') '+------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+'
         do iLev=1,ctlInfo%zDef
            write (lUnit,fmt='("|",F6.1,"|",5(F8.1,":",F8.1,"|"))') ctlInfo%levels(iLev) &
                   , minvalue(1,iLev,iFile),maxvalue(1,iLev,iFile) &
                   , minvalue(2,iLev,iFile),maxvalue(2,iLev,iFile) &
                   , minvalue(3,iLev,iFile),maxvalue(3,iLev,iFile) &
                   , minvalue(4,iLev,iFile),maxvalue(4,iLev,iFile) &
                   , minvalue(5,iLev,iFile),maxvalue(5,iLev,iFile)
         enddo
         write (lUnit,fmt='(A)') '+------+--------+--------+--------+--------+--------+--------+--------+--------+--------+--------+'
      end do

      ierrNumber=releaseUnit(lunit)

      write(cl,fmt='(I3.3)') ctlInfo%zDef-1
      do ivar=1,ctlInfo%nVars
         lunit=getUnit()
         open(unit=lunit, file=trim(ctlInfo%varnames(ivar))//'_min.csv', form='formatted', action='write', status='replace')
         write(lUNit,fmt='("Hr;",'//cl//'(F8.1,";"),F8.1)') ctlInfo%levels(1:ctlInfo%zDef-1),ctlInfo%levels(ctlInfo%zDef)
         do iFile=1,stepsBetDates
            write(lUNit,fmt='(I2.2,";"'//cl//'(F8.1,";"),F8.1)') iFile,minValue(ivar,1:ctlInfo%zDef-1,iFile),minValue(ivar,ctlInfo%zDef,iFile)
         enddo
         ierrNumber=releaseUnit(lunit)
         lunit=getUnit()
         open(unit=lunit, file=trim(ctlInfo%varnames(ivar))//'_max.csv', form='formatted', action='write', status='replace')
         write(lUNit,fmt='("Hr;",'//cl//'(F8.1,";"),F8.1)') ctlInfo%levels(1:ctlInfo%zDef-1),ctlInfo%levels(ctlInfo%zDef)
         do iFile=1,stepsBetDates
            write(lUNit,fmt='(I2.2,";"'//cl//'(F8.1,";"),F8.1)') iFile,maxValue(ivar,1:ctlInfo%zDef-1,iFile),maxValue(ivar,ctlInfo%zDef,iFile)
         enddo
         ierrNumber=releaseUnit(lunit)
      enddo

   end subroutine makeTable 

end module filesMod 